//
//  AudioManager.m
//  GJDigitalDemo
//
//  Created by guiji on 2023/7/17.
//

#import "AudioManager.h"
@interface AudioManager()
{
    CMSampleBufferRef lastSample;
}
@property(nonatomic,strong)NSMutableArray * audioList;
@property (nonatomic, strong)PLAssetReader *assetReader;
@property (nonatomic, assign) CGFloat frameRate;
// 音视频解码时间戳同步
@property (nonatomic, assign) CFAbsoluteTime startActualFrameTime;
@property (nonatomic, assign) NSInteger urlNum;

@end
@implementation AudioManager
+(AudioManager*)manager
{
    static dispatch_once_t once;
    static AudioManager *manager;
    dispatch_once(&once, ^{
        manager = [AudioManager new];
    });
    return manager;
}
-(id)init
{
    self=[super init];
    if(self)
    {
        self.audioList=[[NSMutableArray alloc] init];
    }
    return self;
}
- (void)initWithPath:(NSString*)audioUrl
{

    self.frameRate=25;

    NSURL * url;
    if([audioUrl containsString:@"http"])
    {
        url=[NSURL URLWithString:audioUrl];
    }
    else
    {
        url=[NSURL fileURLWithPath:audioUrl];
    }

    self.assetReader = [[PLAssetReader alloc] initWithURL:url frameRate:self.frameRate stereo:NO];

//    CMTime duration = kCMTimeZero;
//    float fps;
//    [_assetReader getAudioInfo:&duration frameRate:&fps];
//    self.frameRate=fps;
  
    __weak typeof(self) weakSelf = self;
    self.assetReader.readCompeleteBlock = ^{
        weakSelf.urlNum++;
        [weakSelf toAutoPlay:weakSelf.urlNum];
    };
    [self toStartPushBuffer];

}
// 开始推 buffer
-(void)toStartPushBuffer
{
    if (_assetReader)
    {
        NSLog(@"_assetReader");

        [self startPushBuffer:kCMTimeZero];
    }
}

// 开始推 buffer
- (void)startPushBuffer:(CMTime)cm_time
{


    NSLog(@"isRunning");

    [self.assetReader seekTo:cm_time frameRate:_frameRate];
    [self resetTime];
//    if (_assetReader.hasVideo) {
//
//      // [self videoPushProc];
//      // [NSThread detachNewThreadSelector:@selector(videoPushProc) toTarget:self withObject:nil];
//    } else {
//        NSLog(@"[PLStreamViewController] media with no video data!");
//    }
    if (_assetReader.hasAudio) {

        [self audioPushProc];

    } else {
        NSLog(@"[PLStreamViewController] media with no audio data!");
    }
//    [self.lock unlock];
}
- (void)audioPushProc {
//    @autoreleasepool {
//        if (self.isRunning) {

            CMSampleBufferRef sample = [self.assetReader readAudioSampleBuffer];
            if (!sample) {

                [self resetTime];
            }

            if (sample) {
           
//                NSData * auidoData=[self getAudioData:sample];
        
                
                CMTime currentSampleTime = CMSampleBufferGetOutputPresentationTimeStamp(sample);
                CFAbsoluteTime currentActualTime = CFAbsoluteTimeGetCurrent();
                
                CFAbsoluteTime duration = CMTimeGetSeconds(currentSampleTime);
                NSTimeInterval ti=((duration - (currentActualTime - _startActualFrameTime)));
                NSLog(@"startActualFrameTime:%f,%f,%f",duration - (currentActualTime - _startActualFrameTime),ti,currentActualTime);
                
                [[DigitalLiveManager manager] deliverRecordedData:sample];
                if(ti>=0 &&ti<1)
                {
     
                   [self performSelector:@selector(audioPushProc) withObject:self afterDelay:ti];

                  
                }
                else
                {
             
          
                    [self performSelector:@selector(audioPushProc) withObject:self afterDelay:0];


                }
       
//                auidoData=nil;
                CFRelease(sample);
            }
//        }
//    }
}
// 重置时间戳
-(void)resetTime {
   _startActualFrameTime = CFAbsoluteTimeGetCurrent();
}
-(void)toStartWithAudioList:(NSMutableArray*)audioList
{
    
    if(audioList.count==0)
    {
        return;
    }
    self.audioList=audioList;
    self.urlNum=0;
    [self toAutoPlay:self.urlNum];
   
}
-(void)toAutoPlay:(NSInteger)url_num
{
    if(self.audioList.count>url_num)
    {
        NSString * audioUrl=self.audioList[url_num];
        [self initWithPath:audioUrl];
 
    }
    else
    {
        url_num=0;
        [self toAutoPlay:url_num];
    }
}
-(void)toStop
{
    if(self.assetReader!=nil)
    {
        [self.assetReader toCancelRead];
        self.assetReader=nil;
    }
    [NSObject cancelPreviousPerformRequestsWithTarget:self];
}
- (CMSampleBufferRef)bufferConvertAudioBuffer:(void *)buffer frameSize:(int64_t)frameSize {
    AVAudioChannelLayout *chLayout = [[AVAudioChannelLayout alloc] initWithLayoutTag:kAudioChannelLayoutTag_Stereo];
    AVAudioFormat *pcmBuffer = [[AVAudioFormat alloc] initWithCommonFormat:AVAudioPCMFormatFloat32
                                                              sampleRate:48000
                                                              interleaved:NO
                                                            channelLayout:chLayout];

    // 创建一个基于上述配置的帧缓存结构
    AVAudioPCMBuffer *thePCMBuffer = [[AVAudioPCMBuffer alloc] initWithPCMFormat:pcmBuffer frameCapacity:1024];
    thePCMBuffer.frameLength = thePCMBuffer.frameCapacity;

    // 初始化数据区
    for (AVAudioChannelCount ch = 0; ch < pcmBuffer.channelCount; ++ch) {
        memset(thePCMBuffer.floatChannelData[ch], 0, thePCMBuffer.frameLength * pcmBuffer.streamDescription->mBytesPerFrame);
    }

    memcpy(thePCMBuffer.floatChannelData[0], buffer, frameSize);

    AudioStreamBasicDescription asbd = *thePCMBuffer.format.streamDescription;
    CMSampleBufferRef sampleBuffer = NULL;
    CMAudioFormatDescriptionRef format = NULL;

    OSStatus error = CMAudioFormatDescriptionCreate(kCFAllocatorDefault, &asbd, 0, NULL, 0, NULL, NULL, &format);
    if (error != noErr) {
        return nil;
    }

    //CMTime PTS = CMTimeMake([AVAudioTime secondsForHostTime:ATS.hostTime] * ATS.sampleRate, ATS.sampleRate);
    CMTime PTS = CMTimeMake(CACurrentMediaTime()*1000, 1000);
    CMItemCount count = thePCMBuffer.audioBufferList->mBuffers[1].mDataByteSize / asbd.mBytesPerFrame;

    error = CMAudioSampleBufferCreateReadyWithPacketDescriptions(kCFAllocatorDefault,
                                                                 NULL,
                                                                 format,
                                                                 count,
                                                                 PTS,
                                                                 NULL,
                                                                 &sampleBuffer);

    if (error != noErr) {
        CFRelease(format);
        NSLog(@"CMAudioSampleBufferCreateReadyWithPacketDescriptions returned error: %d", (int)error);
        return nil;
    }

    error = CMSampleBufferSetDataBufferFromAudioBufferList(sampleBuffer, kCFAllocatorDefault, kCFAllocatorDefault, kCMSampleBufferFlag_AudioBufferList_Assure16ByteAlignment, thePCMBuffer.audioBufferList);

    if (error != noErr) {
        CFRelease(format);
        NSLog(@"CMSampleBufferSetDataBufferFromAudioBufferList returned error: %d", (int)error);
        return nil;
    }

    CFRelease(format);
    return sampleBuffer;
}
@end
